"""Module level object."""
from .__version__ import __version__
from .__version__ import __description__
from .__version__ import __url__
from .__version__ import __title__
from .__version__ import __author__
from .__version__ import __author_email__
from .__version__ import __license__
from .__version__ import __copyright__


__all__ = (
    __version__,
    __description__,
    __url__,
    __title__,
    __author__,
    __author_email__,
    __license__,
    __copyright__,
)
